/*
* Copyright (C) 2005, Pasi P. Liimatainen
* Distributable under GNU General Public License (v2)
* See terms of license at http://www.gnu.org
*/

//
// User account management.
//
function UserManager(deliciousService)
{
    this.deliciousService = deliciousService;
    this.init();
}

UserManager.prototype = {
	defaultUserKey: 'delicious.users.defaultUser',
	usersKey: 'delicious.users',
	passwordMgmtHost: 'delicious.mozdev.org',
	dummyUser: null,

    init: function() {
        this.users = new Array;
        this.dummyUser = new User("", this.deliciousService);
        this.defaultUser = this.dummyUser;
        var usernames = nsPreferences.copyUnicharPref(this.usersKey, "").split(' ');
        var defUserName = nsPreferences.copyUnicharPref(this.defaultUserKey, "");
        if (usernames[0].length == 0) {
            return;
        }
        for (u in usernames) {
            var user = new User(usernames[u], this.deliciousService);
            this.users.push(user);
            if (defUserName == usernames[u]) {
                this.defaultUser = user;
            }
        }
//        this.importOldAccounts();  // TODO Can be removed once "most people" are running account managed version.
    },

    //
    // Return an array of all users.
    //
    getUsers: function()
    {
        return this.users;
    },

    //
    // Return the named user or null.
    //
    getUserByName: function(username)
    {
        for (user in this.users) {
            if (this.users[user].getUsername() == username) {
                return this.users[user];
            }
        }
        return this.dummyUser;
    },

    //
    // Return the default user.
    //
    getDefaultUser: function()
    {	
        // TODO if there are any problems, check if we should setDefaultUser here, given that the defaultUser is not set
        return this.defaultUser;
    },

    preloadTagsCallback: function(aUserManager) {
      var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      observerService.notifyObservers(null,"ext:delicious:tagxmlForDefaultUser",aUserManager.getDefaultUser().tags.asXml(true));
    },

    //
    // Set the default user.
    //
    setDefaultUser: function(username)
    {
        if (username.length == 0) {
            nsPreferences.setUnicharPref(this.defaultUserKey, username);
            this.defaultUser = this.dummyUser;
            return true;
        }	
        var user = this.getUserByName(username);
        if (user.getPassword().length == 0) {
          alert("The del.icio.us account doesn't have a password defined.\nPlease set the password.");
          return false;
        }
        this.defaultUser = user;
        nsPreferences.setUnicharPref(this.defaultUserKey, username);

        var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        observerService.notifyObservers(null,"ext:delicious:defaultUserChanged",user.getUsername());

        return true;
    },

    //
    // Create a user. Return a string naming the created account.
    //
    createUser: function(username, password, accountname)
    {
        var url = "chrome://delicious/";
        var users = nsPreferences.copyUnicharPref(this.usersKey, "");
        var newUsers = (users + (users.length > 0 ? ' ' : '') + username).split(" ").sort().join(" ");
        nsPreferences.setUnicharPref(this.usersKey, newUsers);
        nsPreferences.setUnicharPref(this.usersKey + '.' + username + ".accountname", accountname);

        if ("@mozilla.org/passwordmanager;1" in Components.classes) {
            var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
            passwordManager.addUser(this.passwordMgmtHost, username, password);
        }
        else if ("@mozilla.org/login-manager;1" in Components.classes) {
            var passwordManager = Components.classes["@mozilla.org/login-manager;1"].getService(Components.interfaces.nsILoginManager);
            var logininfo = Components.classes["@mozilla.org/login-manager/loginInfo;1"].createInstance(Components.interfaces.nsILoginInfo);
            logininfo.init(url, null, this.passwordMgmtHost, username, password, "", "");
            passwordManager.addLogin(logininfo);
        }

        this.users.push(new User(username, this.deliciousService));

        if (users.length == 0) {
            this.setDefaultUser(username);
        }

        return username;
    },

    //
    // Delete the named user.
    //
    deleteUser: function(username)
    {
        var url = "chrome://delicious/";
        var user = this.getUserByName(username);
        if (user.getPassword().length > 0) {
            if ("@mozilla.org/passwordmanager;1" in Components.classes) {
                var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
                passwordManager.removeUser(this.passwordMgmtHost, username);
            }
            else if ("@mozilla.org/login-manager;1" in Components.classes) {
                var passwordManager = Components.classes["@mozilla.org/login-manager;1"].getService(Components.interfaces.nsILoginManager);
                var passwords = passwordManager.findLogins({}, url, null, this.passwordMgmtHost);
                if (passwords.length > 0) {
                    for (var i = 0; i < passwords.length; i++) {
                        if (passwords[i].username == username) {
                          passwordManager.removeLogin(passwords[i]);
                          break;
                        }
                    }
                }
            }
        }

        var usernames = nsPreferences.copyUnicharPref(this.usersKey, "").split(' ');
        for (i in usernames) {
            if (username == usernames[i]) {
                usernames.splice(i, 1);
                break;
            }
        }
        for (i in this.users) {
            if (username == this.users[i].getUsername()) {
                this.users.splice(i, 1);
                break;
            }
        }
        usernames = usernames.join(" ");
        nsPreferences.setUnicharPref(this.usersKey, usernames);
        
        if (this.defaultUser.getUsername() == username) {
            this.setDefaultUser(usernames.split(" ")[0]);
        }

        nsPreferences.mPrefService.deleteBranch(this.usersKey + '.' + username);
    },

    //
    // Import old accounts
    //
    importOldAccounts: function()
    {
        // If accounts haven't been used but there's old (not account managed) delicious
        // user info available, create an account by importing the info and set it as the default account.
        //TODO This code can be removed once "most people" are running account managed version.
        const usernameKey = 'delicious.username';
        const passwordKey = 'delicious.password';
        if (this.getDefaultUser().getUsername().length == 0) {
            var username = nsPreferences.copyUnicharPref(usernameKey, "");
            var password = nsPreferences.copyUnicharPref(passwordKey, "");
            var user = this.createUser(username, password, username);
            this.setDefaultUser(user);

            // Delete the old user preferences.
            nsPreferences.mPrefService.deleteBranch(usernameKey);
            nsPreferences.mPrefService.deleteBranch(passwordKey);
        }
    }
}
